// Sound.cpp: implementation of the CMusic class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "Common.h"
#include "Music.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CMusic::CMusic()
{
    m_lpDMLoader		= NULL;
	m_lpDMPerformance	= NULL;

	m_MusicVolume = 0;
	m_bInit = FALSE;
}

CMusic::~CMusic()
{
	m_bInit = FALSE;
	CoUninitialize();	
}

/*
 *  DM initialization, require window handle 
 */
void	CMusic::CreateDirectMusicSystem(HWND hWnd)
{
	DXTRACE("Init DirectMusic system...");
	DWORD		dwRet;
	//
	// Check initialization of system
	if(m_bInit) {
		DXTHROW("System DirectMusic is already initialized.");
	}
	//
	// 1. COM initialization
	CoInitialize(NULL);
	//
	// 2. Create DMLoader	
	dwRet = CoCreateInstance(CLSID_DirectMusicLoader, NULL, 
							 CLSCTX_INPROC, IID_IDirectMusicLoader8,
							 (void**)&m_lpDMLoader);
	if(dwRet != S_OK) {
		DXTHROWERR("Cannot create DMLoader due ", dwRet);
	}	
	//
	// 3. Create DMPerformance
	dwRet = CoCreateInstance(CLSID_DirectMusicPerformance, NULL,
							 CLSCTX_INPROC, IID_IDirectMusicPerformance8,
							 (void**)&m_lpDMPerformance);
	if(dwRet != S_OK) {
		DXTHROWERR("Cannot create DMPerformance due ", dwRet);
	}	
	//
	// Init DMPerformance
	dwRet = m_lpDMPerformance->InitAudio( 
					NULL,                  // IDirectMusic interface not needed.
					NULL,                  // IDirectSound interface not needed.
					hWnd,                  // Window handle.
					DMUS_APATH_DYNAMIC_STEREO ,  // Default audiopath type.
					64,                    // Number of performance channels.
					DMUS_AUDIOF_ALL,       // Features on synthesizer.
					NULL                   // Audio parameters; use defaults.
				);
	if(dwRet != S_OK) {
		DXTHROWERR("Cannot init DMPerformance due ", dwRet);
	}	
	//
	// Set search directory - default is Music
	CString csPath = setGetDataFilePath("Music");
	dwRet = m_lpDMLoader->SetSearchDirectory(GUID_DirectMusicAllTypes, csPath.AllocSysString(), TRUE);
	if(dwRet != S_OK) {
		DXTHROWERR("Cannot set search directory due ", dwRet);
	}	
	//
	// Initialization OK
	m_bInit = TRUE;	
}

/*
 *	Deinits DM system
 */
void CMusic::DeinitMusic()
{
	DWORD dwRet;
	//
	// Check initialization of system
	if(!m_bInit) {
		DXTHROW("System DirectMusic is not initialized.");
	}	
	//
	// Stop all music and sounds
	if(m_lpDMPerformance) {
		dwRet = m_lpDMPerformance->Stop(
			NULL,   // Stop all segments.
			NULL,   // Stop all segment states.
			0,      // Do it immediately.
			0       // Flags.
		);
		if(dwRet != S_OK) {
			DXTHROWERR("cannot stop all tracks due", dwRet);			
		}
		// Close performance
		m_lpDMPerformance->CloseDown();
	}
	SAudio *pAudio;
	//
	// Delete all segments	
	for(int i = 0; i < m_arSegments.GetSize(); i++) {
		pAudio = (SAudio*) m_arSegments[i];
		SAFE_RELEASE(pAudio->m_lpSegment);
		SAFE_DELETE(pAudio);
	}
	// 
	// Safe release loader and performance
	SAFE_RELEASE(m_lpDMLoader);
	SAFE_RELEASE(m_lpDMPerformance);	
}

/*
 *	Loads music from .mid file or sound from .wav file
 */
void	CMusic::LoadMusicFromFile(DWORD dwID, CString BMPFile)
{
	DWORD dwRet;
	SAudio		*pAudio;
	//
	// Check initialization of system
	if(!m_bInit) {
		DXTHROW("System DirectMusic is not initialized.");
	}	
	//
	// Check if the specified ID is not already in array
	for(int i = 0; i < m_arSegments.GetSize(); i++) {
		pAudio = (SAudio*) m_arSegments[i];
		if(pAudio->m_dwID == dwID) {
			DXTHROW("Segment %d - %s is already in the array.", dwID, BMPFile);
		}
	}
	//
	// Create new audio segment
	SAudio	*pNew = new SAudio;
	pNew->m_dwID = dwID;
	//
	// Try to load audio file
	dwRet = m_lpDMLoader->LoadObjectFromFile(
		CLSID_DirectMusicSegment,    // Class identifier.
		IID_IDirectMusicSegment8,    // ID of desired interface.
		BMPFile.AllocSysString(),    // Filename.
		(LPVOID*) &pNew->m_lpSegment // Pointer that receives interface.
	);
	if(dwRet != S_OK) {
		SAFE_DELETE(pNew);
		DXTHROWERR("Cannot load audio from file due", dwRet);		
	}
	//
	// Download audio to synthesizer 
	dwRet = pNew->m_lpSegment->Download( m_lpDMPerformance );
	if(dwRet != S_OK) {
		SAFE_DELETE(pNew);
		DXTHROWERR("Cannot download audio due", dwRet);				
	}
	//
	// Add audio segment pointer
	m_arSegments.Add(pNew);
}

/*
 * Plays music in the primary buffer
 */
void CMusic::PlayMusic(DWORD dwID)
{
	DWORD dwRet;
	SAudio *pAudio;
	//
	// Check initialization of system
	if(!m_bInit) {
		DXTHROW("System DirectMusic is not initialized.");
	}
	//
	// Find source and start playing	
	for(int i = 0; i < m_arSegments.GetSize(); i++) {
		pAudio = (SAudio*) m_arSegments[i];
		if(pAudio->m_dwID == dwID) {
			//
			// Play segment
			dwRet = m_lpDMPerformance->PlaySegmentEx(
						pAudio->m_lpSegment,	// Segment to play.
						NULL,					// Used for songs; not implemented.
						NULL,					// For transitions. 
						0 ,					    // Flags.
						0,						// Start time; 0 is immediate.
						NULL,					// Pointer that receives segment state.
						NULL,					// Object to stop.
						NULL					// Audiopath, if not default.
					);  
			if(dwRet != S_OK) {
				DXTHROWERR("Cannot play music due", dwRet);					
			}
		}			
	}			
}

/*
 * Plays sound in the secondary buffer
 */
void CMusic::PlaySound(DWORD dwID)
{
	DWORD dwRet;
	SAudio *pAudio;
	//
	// Check initialization of system
	if(!m_bInit) {
		DXTHROW("System DirectMusic is not initialized.");
	}
	//
	// Find source and start playing
	for(int i = 0; i < m_arSegments.GetSize(); i++) {
		pAudio = (SAudio*) m_arSegments[i];
		if(pAudio->m_dwID == dwID) {
			dwRet = m_lpDMPerformance->PlaySegmentEx(
						pAudio->m_lpSegment,	// Segment to play.
						NULL,					// Used for songs; not implemented.
						NULL,					// For transitions. 
						DMUS_SEGF_SECONDARY ,   // Flags.
						0,						// Start time; 0 is immediate.
						NULL,					// Pointer that receives segment state.
						NULL,					// Object to stop.
						NULL					// Audiopath, if not default.
					);  
			if(dwRet != S_OK) {
				DXTHROWERR("Cannot play sound due ", dwRet);					
			}			
		}			
	}		
}


/*
 *	Stop music or sound 
 */
void CMusic::StopMusic(DWORD dwID)
{
	//
	// Check initialization of system
	if(!m_bInit) {
		DXTHROW("System DirectMusic is not initialized.");
	}
	SAudio *pAudio;	
	for(int i = 0; i < m_arSegments.GetSize(); i++) {
		pAudio = (SAudio*) m_arSegments[i];
		if(pAudio->m_dwID == dwID) {
			m_lpDMPerformance->Stop(pAudio->m_lpSegment, NULL , 0, 0);
		}
	}	
}

void CMusic::IncreaseMusicVolume()
{
	//
	// Check initialization of system
	if(!m_bInit) {
		DXTHROW("System DirectMusic is not initialized.");
	}
	int Volume = m_MusicVolume;
	//
	// Increment volume
	Volume += 100;
	// Set 0 if 0
	if(Volume >= 0)
		Volume = 0;
	// Set volme to performance
	m_lpDMPerformance->SetGlobalParam( GUID_PerfMasterVolume, 
										   (void*)&Volume, sizeof(long) ); 
	// Remember volume
	m_MusicVolume = Volume;		
}

void CMusic::DecreaseMusicVolume()
{
	//
	// Check initialization of system
	if(!m_bInit) {
		DXTHROW("System DirectMusic is not initialized.");
	}
	//
	// Increment volume
	int Volume = m_MusicVolume;
	Volume -= 100;
	// Set 0 if 0
	if(Volume >= 0)
		Volume = 0;
	// Set volme to performance
	m_lpDMPerformance->SetGlobalParam( GUID_PerfMasterVolume, 
										   (void*)&Volume, sizeof(long) ); 
	// Remember volume
	m_MusicVolume = Volume;
}




